#!/bin/bash

#  lbd.sh (Little Build Daemon)
#  This shell script rebuilds distributions of (no)src.rpm files
#
#  Copyright (c) by Stefan van der Eijk <stefan@eijk.nu>
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
#
#  Wed Sep 06 2006 Stefan van der Eijk <stefan@eijk.nu> 0.25-11
#  - workaround for broken rpm --root implementation
#  - move searching lockfile from ${LBD}/${P_ARCH}/searching
#    to ${BUILDOUTPUT}/searching
#  - change sleep value from 5 to 1
#
#  Sat Jul 15 2006 Stefan van der Eijk <stefan@eijk.nu> 0.25-10
#  - use --strict-arch with urpmi (slower, but better)
#
#  2006/07/13 Stefan van der Eijk <stefan@eijk.nu>
#  - find_new(): add -maxdepth to find in ${RPM_SRC}
#
#  2006/07/12 Stefan van der Eijk <stefan@eijk.nu>
#  - add extra step: dependency. Build order is now:
#    rebuild, new, dependency, old
#    - added find_dependency()
#    - removed finding the dependency rebuilds from find_old()
#  - Don't copy the buildoutput to a seperate directory when
#    a segfault / internal compiler error takes place.
#    Adding a comment to the logfile is sufficient.
#  - reworked BuildRequires removal
#  - removed confusing comment from update_media()
#
#  2006/06/11 Stefan van der Eijk <stefan@eijk.nu>
#  - build:
#    - BUILD_RESULT is unknow then there must be a problem
#  - check_config:
#    - add checking global variables
#    - when an error is found, remove the runfile
#  - cleanup:
#    - make simpler
#  - main:
#    - make configuration file configurable (${config_file})
#
# 2006/06/07 Stefan van der Eijk <stefan@eijk.nu>
#  - improvements to make the lbd homedir share-able over multiple machines
#  - check for "no package named" --> indication that something is wrong
#    with the chroot
#  - rewrote BuildRequires removal
#  - turn on verbose logging with removal of BuildRequires (for now)
#
#  2006/05/09 Stefan van der Eijk <stefan@eijk.nu>
#  - Unmount /proc, /dev, /dev/pts, /sys & /proc/bus/usb after rpms
#    are removed from chroot
#
#  2006/05/03 Stefan van der Eijk <stefan@eijk.nu>
#  - rewrote find_old so the oldest build is rebuilt
#
#  2005/12/24 Stefan van der Eijk <stefan@eijk.nu>
#  - add RPM_TMPPATH and RPM_TOPDIR variables
#  - remove commentd out code
#  - fix bug in update_media(), media are now updated quicker
#
#  2005/12/24 Stefan van der Eijk <stefan@eijk.nu>
#  - use urpmdiff to determin differences in packages, this now replaces
#    the files, provides and requires checks
#  - remove the obsolete STRACE option
#
#  2005/12/15 Stefan van der Eijk <stefan@eijk.nu>
#  - add ability to update multiple chroots with one media
#
#  2005/10/29 Stefan van der Eijk <stefan@eijk.nu>
#  - added find_rebuild()
#  - fixed removing binary packages
#  - build src.rpm in the chroot and not on the native system
#
#  2005/08/16 Stefan van der Eijk <stefan@eijk.nu>
#  - fixed bug in Provides & Requires checking
#
#  2005/08/07 Stefan van der Eijk <stefan@eijk.nu>
#  - add check_config
#  - don't use update_chroot as it messes up the order in which the chroots
#    are updated, same applies to REGENERATE
#  - add switch to not build src.rpm's that produce noarch.rpm packages.
#    To build these packages add "BUILD_NOARCH=1" to configuration of 
#    that source.
#  - add date to filename of logfile
#  - merge urpmi and urpmi_error output files into buildoutput file
#  - mount /dev/pts and /proc from the native system to the chroot
#    before building, unmount them after building
#  - replaced __KEEP_BUILDDIR construction with KEEP_BUILDDIR=2
#  - instroduced BUILD_RESULT to be able to determin where the buildoutput will
#    be put, then remove the buildrequires, then actually move the buildoutput
#  - remove min() --> it wasn't used anymore
#  - renamed urpmi_media() to update_media()
#  - renmaed "urpmi" variable to "media" to be a bit smart friendlier
#  - revert from $OLD_TARGET --> not documented
#  - some other misc changes
#
#  2005/05/12 Christiaan Welvaart <cjw@daneel.dyndns.org>
#  - new function update_chroot
#    separate updating of media and upgrading of chroots
#  - new options:
#    * FORCE_KEEP_BUILDDIR
#      save build dir(s) even if build succeeds
#    * REGENERATE (for medium)
#       medium is an rpm dir; rebuild hdlist when necessary
#    * UPDATE (for target)
#      update the chroot for this target (using the target's media list)
#    * BUILD_OLD
#      set to "no" to only build new packages
#
#  2005/05/11 Stefan van der Eijk <stefan@eijk.nu>
#  - also move noarch.rpm packages
#  - disable min() routine
#
#  2005/03/28 Stefan van der Eijk <stefan@eijk.nu>
#  - fix smart support
#
#  2005/03/10 Stefan van der Eijk <stefan@eijk.nu>
#  - add srpms_mismatch check
#  - sleep 5
#
#  2005/03/07 Stefan van der Eijk <stefan@eijk.nu>
#  - changed ${SLBD} to ${LBD}
#  - sort_on_buildtime (Christiaan Welvaart)
#  - misc apt changes (Christiaan Welvaart)
#
#  2005/03/01 Stefan van der Eijk <stefan@eijk.nu>
#  - renamed config file from ~/.slbd to ~/.lbd
#
#  2005/03/01 Stefan van der Eijk <stefan@eijk.nu>
#  - use same date format
#  - detect dependency loop
#  - add multiarch check
#  - add smart support
#
#  2004/10/01 Stefan van der Eijk <stefan@eijk.nu>
#  - Also switch .rpmmacros file for lbd user on the native system.
#    It was already being switched in the chroot. This is needed for
#    building the src.rpm from where the BuildRequires will be installed.
#
#  v0.27 2004/08/03
#  - check to see if other hosts aren't searching for new packages to build
#
#  v0.26 2004/07/29
#  - add support for nosrc.rpm packages
#
#  v0.25 2004/07/21
#  - add KEEP_BUILDDIR variable, to save the contents of the builddir
#  - add rpmlint checks
#
#  v0.24 2004/07/11
#  - Misc fedora changes
#
#  v0.23 2004/06/18
#  - added filter for "Internal compiler error"
#
#  v0.22 2004/05/21
#  - put segfault buildoutputs in a seperate dir
#  - remove CHROOT=0 and CHROOT variable, chroots are always used
#  - introduce INSTALLER variable
#
#  v0.21 2004/04/30
#  - introduces RPM_OPTIONS variable, to cope with fedora's rpmbuild
#
#  v0.20 2004/04/18
#  - make paths to executables variable
#  - changed the name of the original ${RPM_BIN} variable to ${COMPARE}
#  - make path to slbd base variable ${SLBD}
#
#  v0.19 2004/04/12
#  - sort out packages with dependency issues
#  - add missing dependency to buildoutput
#
#  v.018 2004/01/05
#  - invoke urpmi with "--keep" option when updating a built environment
#
#  v.017 2004/01/02
#  - only check for Segmentation fault on problem packages
#
#  v0.16 2003/12/23
#  - fixed hardcoded /chroot/cooker --> ${CHROOT_PATH} (vdanen)
#  - use unified diff for Filelist, Provides and Requires comparison
#
#  v0.15 2003/12/15
#  - put the Segfault check before the other BuildOutput checks
#
#  v0.14 2003/11/26
#  - changed the direcotry structure
#
#  v0.13 2003/11/22
#  - added rpm_qa directories to contain package listing at build time
#
#  v0.12 2003/11/18
#  - added filter for "Segmentaion fault"
#
#  v0.11 2003/11/13
#  - rebuild src.rpm to obtain correct dependency information
#
#  v0.10 2003/10/31
#  - add KEEP_RPMS
#
#  v0.09 2003/10/18
#  - invoke urpmi with --no-uninstall
#
#  v0.08 2003/09/07
#  - make automake1.7 hack simpler
#  - added "File not found" check
#
#  v0.07 2003/09/02
#  - quite some small fixes
#  - automake1.7 hack
#
#  v0.06 2003/07/06
#  - added support for:
#    - configuration file
#    - multiple instances on one machine (different chroots)
#  - merged min.sh
#
#  v0.05 2003/05/26
#  - implemented chroot support
#  - rewrite of find_new() and find_old()
#
#  v0.04 2003/05/02
#  - different urpmi locations per arch
#  - delete only ${HOSTTYPE} packages, not noarch
#
#  v0.03 2003/04/27
#  - add number of packages installed to the log
#  - log when min.sh is run
#
#  v0.03 2003/04/22
#  - log what packages rebuilt
#  - try to recover initial state by running min.sh
#
#  v0.02 2003/04/22
#  - check if number of packages is the same before & after build
#  otherwise remove /build/run/`hostname -f` file
#  - invoke urpmi with -q option
#
#  v0.01 2003/04/06
#  - first release under the GPL
#

export PATH=$PATH:/usr/X11R6/bin:/usr/sbin:/sbin

check_config () {

ERROR=0

. ${config_file}

echo "Checking global variables"
if	[ "${CHROOT_BIN}" != "" ] \
	|| [ -x "${CHROOT_BIN}" ] ; then
	echo "Required Global parameter \"CHROOT_BIN\" set to \"${CHROOT_BIN}\""
else
	echo "Required Global parameter \"CHROOT_BIN\" not set, or file doesn't exist - ERROR" ; ERROR=1
fi
if	[ "${INSTALLER}" = "apt" ]; then
	echo "Required Global parameter \"INSTALLER\" set to \"${INSTALLER}\""
	if	[ "${APT_BIN}" != "" ] \
		|| [ -x "${APT_BIN}" ] ; then
		echo "Required Global parameter \"APT_BIN\" set to \"${APT_BIN}\""
	else
		echo "Required Global parameter \"APT_BIN\" not set, or file doesn't exist - ERROR"; ERROR=1
	fi
elif	[ "${INSTALLER}" = "smart" ]; then
	echo "Required Global parameter \"INSTALLER\" set to \"${INSTALLER}\""
	if	[ "${SMART_BIN}" != "" ] \
		|| [ -x "${SMART_BIN}" ] ; then
		echo "Required Global parameter \"SMART_BIN\" set to \"${SMART_BIN}\""
	else
		echo "Required Global parameter \"SMART_BIN\" not set, or file doesn't exist - ERROR"; ERROR=1
	fi
elif	[ "${INSTALLER}" = "urpmi" ]; then
	echo "Required Global parameter \"INSTALLER\" set to \"${INSTALLER}\""
	if	[ "${URPMI_BIN}" != "" ] \
		|| [ -x "${URPMI_BIN}" ] ; then
		echo "Required Global parameter \"URPMI_BIN\" set to \"${URPMI_BIN}\""
	else
		echo "Required Global parameter \"URPMI_BIN\" not set, or file doesn't exist - ERROR"; ERROR=1
	fi
	if	[ "${URPMI_UPDATE_BIN}" != "" ] \
		|| [ -x "${URPMI_UPDATE_BIN}" ] ; then
		echo "Required Global parameter \"URPMI_BIN\" set to \"${URPMI_UPDATE_BIN}\""
	else
		echo "Required Global parameter \"URPMI_BIN\" not set, or file doesn't exist - ERROR"; ERROR=1
	fi
elif	[ "${INSTALLER}" = "yum" ]; then
	echo "Required Global parameter \"INSTALLER\" set to \"${INSTALLER}\""
	if	[ "${YUM_BIN}" != "" ] \
		|| [ -x "${YUM_BIN}" ] ; then
		echo "Required Global parameter \"YUM_BIN\" set to \"${YUM_BIN}\""
	else
		echo "Required Global parameter \"YUM_BIN\" not set, or file doesn't exist - ERROR"; ERROR=1
	fi
	if	[ "${YUM_CONFIG}" != "" ] \
		|| [ -f "${YUM_CONFIG}" ] ; then
		echo "Required Global parameter \"YUM_CONFIG\" set to \"${YUM_CONFIG}\""
	else
		echo "Required Global parameter \"YUM_CONFIG\" not set, or file doesn't exist - ERROR"; ERROR=1
	fi
else
	echo "Required Global parameter \"INSTALLER\" not set, or set to an unkown value - ERROR"; ERROR=1
fi
if	[ "${RPM_BIN}" != "" ] \
	|| [ -x "${RPM_BIN}" ] ; then
	echo "Required Global parameter \"RPM_BIN\" set to \"${RPM_BIN}\""
else
	echo "Required Global parameter \"RPM_BIN\" not set, or file doesn't exist - ERROR"; ERROR=1
fi
if	[ "${RPM_REBUILD_BIN}" != "" ] \
	|| [ -x "${RPM_REBUILD_BIN}" ] ; then
	echo "Required Global parameter \"RPM_REBUILD_BIN\" set to \"${RPM_REBUILD_BIN}\""
else
	echo "Required Global parameter \"RPM_REBUILD_BIN\" not set, or file doesn't exist - ERROR"; ERROR=1
fi
if	[ "${LBD}" != "" ] \
	|| [ -d "${LBD}" ] ; then
	echo "Required Global parameter \"LBD\" set to \"${LBD}\""
else
	echo "Required Global parameter \"LBD\" not set, or directory doesn't exist - ERROR"; ERROR=1
fi
if	[ "${SUDO_BIN}" != "" ] \
	|| [ -x "${SUDO_BIN}" ] ; then
	echo "Required Global parameter \"SUDO_BIN\" set to \"${SUDO_BIN}\""
else
	echo "Required Global parameter \"SUDO_BIN\" not set, or file doesn't exist - ERROR"; ERROR=1
fi

echo "Checking sources"
for a in `${config_file} src`; do
	. ${config_file} ${a}
	if [ "${BUILD_NOARCH}" = "" ]; then
		echo "${a} Optional parameter \"BUILD_NOARCH\" not set"
	elif [ "${BUILD_NOARCH}" = "1" ]; then
		echo "${a} Optional parameter \"BUILD_NOARCH\" set to \"1\""
	else
		echo "${a} Optional parameter \"BUILD_NOARCH\" set to \"${BUILD_NOARCH}\" - ERROR"; ERROR=1
	fi
	if [ "${BUILD_OUTPUT}" = "" ]; then
		echo "${a} Required parameter \"BUILD_OUTPUT\" not set - ERROR"; ERROR=1
	elif [ -d ${BUILD_OUTPUT} ]; then
		echo "${a} Required parameter \"BUILD_OUTPUT\": directory \"${BUILD_OUTPUT}\" exists"
	else
		echo "${a} Required parameter \"BUILD_OUTPUT\": directory \"${BUILD_OUTPUT}\" doesn't exist - ERROR"; ERROR=1
	fi
	if [ "${CHROOT_PATH}" = "" ]; then
		echo "${a} Required parameter \"CHROOT_PATH\" not set - ERROR"; ERROR=1
	elif [ -d ${CHROOT_PATH} ]; then
		echo "${a} Required parameter \"CHROOT_PATH\": directory \"${CHROOT_PATH}\" exists"
	else
		echo "${a} Required parameter \"CHROOT_PATH\": directory \"${CHROOT_PATH}\" doesn't exist - ERROR"; ERROR=1
	fi
	if [ "${COMPARE}" = "" ]; then
		echo "${a} Optional parameter \"COMPARE\" not set"
	elif [ -d ${COMPARE} ]; then
		echo "${a} Optional parameter \"COMPARE\": directory \"${COMPARE}\" exists"
	else
		echo "${a} Optional parameter \"COMPARE\": directory \"${COMPARE}\" doesn't exist - ERROR"; ERROR=1
	fi
	if [ "${KEEP_BUILDDIR}" = "" ]; then
		echo "${a} Optional parameter \"KEEP_BUILDDIR\" not set"
	elif [ "${KEEP_BUILDDIR}" = "1" ]; then
		echo "${a} Optional parameter \"KEEP_BUILDDIR\" set to \"1\""
	elif [ "${KEEP_BUILDDIR}" = "2" ]; then
		echo "${a} Optional parameter \"KEEP_BUILDDIR\" set to \"2\""
	else
		echo "${a} Optional parameter \"KEEP_BUILDDIR\" set to an unkown value - ERROR"; ERROR=1
	fi
	if [ "${KEEP_RPMS}" = "" ]; then
		echo "${a} Optional parameter \"KEEP_RPMS\" not set"
	elif [ "${KEEP_RPMS}" = "1" ]; then
		echo "${a} Optional parameter \"KEEP_RPMS\" set to \"1\""
	else
		echo "${a} Optional parameter \"KEEP_RPMS\" set to an unkown value - ERROR"; ERROR=1
	fi
# Todo for ${MEDIA}, check if it exists in apt / smart / urpmi 
	if [ "${MEDIA}" = "" ]; then
		echo "${a} Required parameter \"MEDIA\" not set - ERROR"; ERROR=1
	else
		for b in `echo ${MEDIA} | perl -p -e "s#,#\n#g" 2> /dev/null`; do
			. ${config_file} media_${b}
			if [ "${MEDIA_PATH}" = "" ]; then
				echo "${b} Required parameter \"MEDIA_PATH\" for MEDIA \"${b}\" not set - ERROR"; ERROR=1
			elif [ -e ${MEDIA_PATH} ]; then
				echo "${b} Required parameter \"MEDIA_PATH\" for MEDIA \"${b}\": \"${MEDIA_PATH}\" exists"
			else
				echo "${b} Required parameter \"MEDIA_PATH\" for MEDIA \"${b}\": \"${MEDIA_PATH}\" doesn't exist - ERROR"; ERROR=1
			fi
		done
	fi
	if [ "${MTIME}" = "" ]; then
		echo "${a} Required parameter \"MTIME\" not set - ERROR"; ERROR=1
	else
		echo "${a} Required parameter \"MTIME\" set to \"${MTIME}\""
	fi
	if [ "${REDUNDANT}" = "" ]; then
		echo "${a} Optional parameter \"REDUNDANT\" not set"
	elif [ "${REDUNDANT}" = "1" ]; then
		echo "${a} Optional parameter \"REDUNDANT\" set to \"1\""
	else
		echo "${a} Optional parameter \"REDUNDANT\" or set to an unkown value - ERROR"; ERROR=1
	fi
	if [ "${RPM_MACROS}" = "" ]; then
		echo "${a} Required parameter \"RPM_MACROS\" not set - ERROR"; ERROR=1
	elif [ -f ${HOME}/${RPM_MACROS} ]; then
		echo "${a} Required parameter \"RPM_MACROS\": file \"${HOME}/${RPM_MACROS}\" exists"
	else
		echo "${a} Required parameter \"RPM_MACROS\": file \"${HOME}/${RPM_MACROS}\" doesn't exist - ERROR"; ERROR=1
	fi
	if [ "${RPM_OPTIONS}" = "" ]; then
		echo "${a} Optional parameter \"RPM_OPTIONS\" not set"
	else
		echo "${a} Optional parameter \"RPM_OPTIONS\" set to \"${RPM_OPTIONS}\""
	fi
	if [ "${RPM_SRC}" = "" ]; then
		echo "${a} Required parameter \"RPM_SRC\" not set - ERROR"; ERROR=1
	elif [ -d ${RPM_SRC} ]; then
		echo "${a} Required parameter \"RPM_SRC\": directory \"${RPM_SRC}\" exists"
	else
		echo "${a} Required parameter \"RPM_SRC\": directory \"${RPM_SRC}\" doesn't exist - ERROR"; ERROR=1
	fi
done

if	[ "${ERROR}" = "1" ]; then
	echo "*** ERROR in configuration file \"${config_file}\", removing \"${LBD}/run/${LOGNAME}@`hostname -f`\" ***"
	rm -v ${LBD}/${LOGNAME}@`hostname -f`
else
	echo "*** Configuration file \"${config_file}\" fine ***"
fi
}

cleanup () {
# Cleanup build environment
# Input:
# - CHROOT_PATH (from parent)
# - P_ARCH (from parent)
# - RPM_TOPDIR (from parent)
# - RPM_TMPPATH (from parent)

echo "%%% Cleanup %%%"

chmod 777 ${CHROOT_PATH}${RPM_TOPDIR}/BUILD/* -R 
rm -fr	${CHROOT_PATH}/tmp/*
rm -fr	${CHROOT_PATH}/var/tmp/*
rm -fr	${CHROOT_PATH}${RPM_TOPDIR}/BUILD/*
rm -fr	${CHROOT_PATH}${RPM_TOPDIR}/RPMS/${P_ARCH}/*
rm -fr	${CHROOT_PATH}${RPM_TOPDIR}/RPMS/noarch/*
rm -fr	${CHROOT_PATH}${RPM_TOPDIR}/SOURCES/*
rm -fr	${CHROOT_PATH}${RPM_TOPDIR}/SPECS/*
rm -fr	${CHROOT_PATH}${RPM_TMPPATH}/*
find	${CHROOT_PATH}${RPM_TOPDIR}/ -type f -exec rm -fr {} \;
}

find_dependency () {
# Find a (no)src.rpm which needs to be rebuilt.
# Input: src name
# - ${1} = src name
# Output: name of (no)src.rpm
# Configuration file: ${config_file}
# - BUILD_OUTPUT
# - LBD
# - P_ARCH

src=${1}

. ${config_file} ${src}

while [ -f ${BUILD_OUTPUT}/.searching ]; do
	sleep 1
done

echo "`hostname -f`" > ${BUILD_OUTPUT}/.searching

RESULT=`find \
${BUILD_OUTPUT}/BO/ \
${BUILD_OUTPUT}/BO/dependency \
-maxdepth 1 \
-mtime +1 \
-type f \
| perl -p -e "s#.*/##g" 2> /dev/null \
| sort -u \
| head -n1`

if [ "${RESULT}" = "" ]; then
	rm -fr \
	${BUILD_OUTPUT}/.searching \
	2> /dev/null
else
	echo "${RESULT}"
fi
}

find_new () {
# Find a new (no)src.rpm to build.
# Input: src name
# - ${1} = src name
# Output: name of (no)src.rpm
# Configuration file: ${config_file}
# - BUILD_OUTPUT
# - LBD
# - P_ARCH
# - RPM_SRC
# - SORT_ON_BUILDTIME (optional)

src=${1}

. ${config_file} ${src}

while [ -f ${BUILD_OUTPUT}/.searching ]; do
	sleep 1
done
echo "`hostname -f`" > ${BUILD_OUTPUT}/.searching

find ${BUILD_OUTPUT}/BO/ -type f \
| perl -p -e "s#.*/##g" 2> /dev/null \
| sort -u \
> ${HOME}/tmp/LOG1.`hostname -f`

find ${BUILD_OUTPUT}/ -type f \
| grep -v .searching \
| grep -v RPMS \
| grep -v KEEP_BUILDDIR \
| perl -p -e "s#.*/##g" 2> /dev/null \
| sort -u \
> ${HOME}/tmp/LOG2.`hostname -f`

find ${RPM_SRC}/ -maxdepth 1 -type f \
| grep src.rpm$ \
| perl -p -e "s#.*/##g" 2> /dev/null \
| perl -p -e "s#^\..*\n##g" 2> /dev/null \
| perl -p -e "s#\.(nosrc|src)\.rpm##g" 2> /dev/null \
| sort -u \
> ${HOME}/tmp/LOG3.`hostname -f`

NEW_FILE="${HOME}/tmp/NEW.`hostname -f`"

diff	${HOME}/tmp/LOG1.`hostname -f` \
	${HOME}/tmp/LOG2.`hostname -f` \
	> ${NEW_FILE}

diff	${HOME}/tmp/LOG1.`hostname -f` \
	${HOME}/tmp/LOG3.`hostname -f` \
	>> ${NEW_FILE}

diff	${HOME}/tmp/LOG2.`hostname -f` \
	${HOME}/tmp/LOG3.`hostname -f` \
	>> ${NEW_FILE}

if [ "${SORT_ON_BUILDTIME}" = "1" ] && [ -x `dirname $0`/sort_on_buildtime ]; then
# FYI: old logs are not removed in this mode
	touch ${BUILD_OUTPUT}/SRPMS.queue
	QUEUESIZE=`wc -l ${BUILD_OUTPUT}/SRPMS.queue | cut -d\  -f1`
	if [ ! \( ${QUEUESIZE} -gt 20 \) ]; then
		cat ${NEW_FILE} \
		| grep "[>|<]" \
		| perl -p -e "s#\\S\\s+(\\S+)#\\1#g" 2> /dev/null \
		| `dirname ${0}`/sort_on_buildtime ${RPM_SRC} 2>/dev/null \
		| perl -p -e "s#.*/##g" 2> /dev/null \
		| perl -p -e "s#^\..*\n##g" 2> /dev/null \
		| perl -p -e "s#\.src\.rpm##g" 2> /dev/null \
		> ${BUILD_OUTPUT}/SRPMS.queue

		QUEUESIZE=`wc -l ${BUILD_OUTPUT}/SRPMS.queue | cut -d\  -f1`
	fi

	RESULT=`head -n 1 ${BUILD_OUTPUT}/SRPMS.queue`
	tail -n `expr ${QUEUESIZE} - 1` ${BUILD_OUTPUT}/SRPMS.queue >${BUILD_OUTPUT}/SRPMS.queue.new
	mv -f ${BUILD_OUTPUT}/SRPMS.queue.new ${BUILD_OUTPUT}/SRPMS.queue
else
	RESULT=`cat ${NEW_FILE} \
	| grep "[>|<]" \
	| perl -p -e "s#\S\s+(\S+)#\1#g" 2> /dev/null \
	| sort -u \
	| head -n 1`
fi

if [ "${RESULT}" = "" ]; then
	rm -fr \
	${HOME}/tmp/LOG?.`hostname -f` \
	${NEW_FILE} \
	${BUILD_OUTPUT}/.searching \
	2> /dev/null
else
	rm -fr \
	${HOME}/tmp/LOG?.`hostname -f` \
	${NEW_FILE} \
	2> /dev/null
	echo "${RESULT}"
fi
unset NEW_FILE
}

find_old () {
# Find an old (no)src.rpm to build.
# Input: src name
# - ${1} = src name
# Output: name of (no)src.rpm
# Configuration file: ${config_file}
# - BUILD_OUTPUT
# - LBD
# - MTIME
# - P_ARCH

src=${1}

. ${config_file} ${src}

while [ -f ${BUILD_OUTPUT}/.searching ]; do
	sleep 1
done
echo "`hostname -f`" > ${BUILD_OUTPUT}/.searching

OLD_FILE="${HOME}/tmp/OLD.`hostname -f`"

#find \
#${BUILD_OUTPUT}/BO/ \
#${BUILD_OUTPUT}/BO/dependency/ \
#-maxdepth 1 \
#-mtime +1 \
#-type f \
#-exec stat -c %Y-%n {} \; \
#> ${OLD_FILE}

find \
${BUILD_OUTPUT}/BO/cannot_be_installed/ \
${BUILD_OUTPUT}/BO/debug/ \
${BUILD_OUTPUT}/BO/file_not_found/ \
${BUILD_OUTPUT}/BO/multiarch/ \
${BUILD_OUTPUT}/BO/noarch/ \
${BUILD_OUTPUT}/BO/OK/ \
${BUILD_OUTPUT}/BO/problem/ \
${BUILD_OUTPUT}/BO/rpm_order/ \
${BUILD_OUTPUT}/BO/unpackaged_files/ \
-mtime ${MTIME} \
-type f \
-exec stat -c %Y-%n {} \; \
> ${OLD_FILE}

RESULT=`sort -u ${OLD_FILE} \
	| head -n 1 \
	| perl -p -e "s#.*/(\S+)#\1#g" 2> /dev/null`

if [ "${RESULT}" = "" ]; then
	rm -fr \
	${BUILD_OUTPUT}/.searching \
	${OLD_FILE} \
	2> /dev/null
else
	echo "${RESULT}"
	rm -fr \
	${OLD_FILE} \
	2> /dev/null
fi
unset OLD_FILE
}

find_rebuild () {
# Find a (no)src.rpm which needs to be rebuilt.
# Input: src name
# - ${1} = src name
# Output: name of (no)src.rpm
# Configuration file: ${config_file}
# - BUILD_OUTPUT
# - LBD
# - P_ARCH

src=${1}

. ${config_file} ${src}

while [ -f ${BUILD_OUTPUT}/.searching ]; do
	sleep 1
done

echo "`hostname -f`" > ${BUILD_OUTPUT}/.searching

RESULT=`find ${BUILD_OUTPUT}/BO/rebuild -type f \
| perl -p -e "s#.*/##g" 2> /dev/null \
| sort -u \
| head -n1`

if [ "${RESULT}" = "" ]; then
	rm -fr \
	${BUILD_OUTPUT}/.searching \
	2> /dev/null
else
	echo "${RESULT}"
fi
}

build () {
# Build a package
# Input:
# - ${1} = package name
# - ${2} = src name
# Output: none
# Configuration file: ${config_file}
# - BUILD_OUTPUT	- required
# - CHROOT_PATH		- required
# - COMPARE		- optional
# - INSTALLER		- required
# - KEEP_BUILDDIR	- optional
# - KEEP_RPMS		- optional
# - MEDIA		- required
# - RPM_MACROS		- required
# - RPM_SRC		- required

pkg=${1}
src=${2}

. ${config_file} ${src}

echo "*** Going for: i=${i} new_old=${new_old} P_ARCH=${P_ARCH} pkg=${pkg} src=${src} ***"

# The previously build rpms are removed from the ${BUILD_OUTPUT}/RPMS/ directory.
for i in `find ${BUILD_OUTPUT}/BO/ \
	-name ${pkg} \
	-exec fgrep "Wrote:" {} \; \
	| perl -p -e "s#.*/##g"`; do
	if [ -e ${BUILD_OUTPUT}/RPMS/${i} ]; then
		echo "`date -R` `rm -frv ${BUILD_OUTPUT}/RPMS/${i}`" \
		>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
	fi
done
#fi

# The old buildoutputs, etc are removed.
find ${BUILD_OUTPUT}/ -name ${pkg} -type f \
| grep -v RPMS \
| xargs rm -fr

# The old KEEP_BUILDDIR tarball is removed.
if [ -f ${BUILD_OUTPUT}/KEEP_BUILDDIR/${pkg}.tar.bz2 ]; then
	rm -fr ${BUILD_OUTPUT}/KEEP_BUILDDIR/${pkg}.tar.bz2
fi

# Test if ${RPM_SRC}/${pkg}.(no)src.rpm exists, if the file doesn't exist, 
# no need to rebuild.
unset SRC_EXT
if	[ -e ${RPM_SRC}/${pkg}.nosrc.rpm ]; then
	SRC_EXT=nosrc
elif	[ -e ${RPM_SRC}/${pkg}.src.rpm ]; then
	SRC_EXT=src
fi

if	[ "${SRC_EXT}" = "nosrc" ] \
	|| [ "${SRC_EXT}" = "src" ]; then

# Check if src.rpm will product noarch packages
# if BUILD_NOARCH is not set to "1", touch the buildoutput file
# and don't rebuild the package
	if	[ `rpm -qp --qf %{BUILDARCHS} ${RPM_SRC}/${pkg}.${SRC_EXT}.rpm` = "noarch" ] \
		&& [ "${BUILD_NOARCH}" != "1" ]; then
		touch ${BUILD_OUTPUT}/BO/noarch/${pkg}
		rm -fr ${BUILD_OUTPUT}/.searching
		continue 3 
	fi

# Flip & copy over the .rpmmacros files
	rm -f ${CHROOT_PATH}${HOME}/.rpmmacros
	cp ${HOME}/${RPM_MACROS} ${CHROOT_PATH}${HOME}/.rpmmacros

RPM_TMPPATH=`${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
	su -l ${LOGNAME} \
	-c "${RPM_BIN} --eval %_tmppath"`
RPM_TOPDIR=`${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
	su -l ${LOGNAME} \
	-c "${RPM_BIN} --eval %_topdir"`

# Mount /proc, /dev, /dev/pts, /sys & /proc/bus/usb
${SUDO_BIN} /bin/mount -t proc none ${CHROOT_PATH}/proc
${SUDO_BIN} /bin/mount --bind /dev ${CHROOT_PATH}/dev
${SUDO_BIN} /bin/mount -t devpts none ${CHROOT_PATH}/dev/pts
${SUDO_BIN} /bin/mount -t sysfs none ${CHROOT_PATH}/sys
${SUDO_BIN} /bin/mount -t usbfs none ${CHROOT_PATH}/proc/bus/usb

# Install BuildRequires
	hostname -f > ${BUILD_OUTPUT}/BO/${pkg}
	rm -fr ${BUILD_OUTPUT}/.searching

	BEGIN_FILE="${HOME}/tmp/BEGIN.`hostname -f`.txt"
	${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
		su -l ${LOGNAME} \
		-c "${RPM_BIN} -qa | sort" \
		> ${BEGIN_FILE}
	begin=`cat ${BEGIN_FILE} | wc -l`

	echo "`date -R` ${new_old} ${src} ${pkg} ${begin}" \
		>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt

# Copy the src.rpm to the chroot, as that is where the action takes place.
	cp -v ${RPM_SRC}/${pkg}.${SRC_EXT}.rpm \
	${CHROOT_PATH}/var/tmp/

# Dependency information is stored in the headers of a rpm package, and are generated
# when the package is built. To get the correct BuildRequires for the (no)src.rpm to be
# rebuilt, a new (no)src.rpm needs to be built, on the correct platform and with the correct
# (build) options specified.
	echo "*** Building ${new_old} (no)src.rpm ***"
	SPEC=`${RPM_BIN} -qpl ${CHROOT_PATH}/var/tmp/${pkg}.${SRC_EXT}.rpm 2> /dev/null \
		| grep "\.spec$"`

	${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
		su -l ${LOGNAME} \
		-c "${RPM_BIN} -i \
		/var/tmp/${pkg}.${SRC_EXT}.rpm" \
		2> /dev/null

# In case of a nosrc.rpm package, copy the missing sourcefiles to the SOURCES directories
# of the lbd homedir on the native system and the chroot.
	if [ "${SRC_EXT}" = "nosrc" ]; then
		unset EXIT
		NOSRC_FILE=`${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
	       		su -l ${LOGNAME} \
			-c "${RPM_REBUILD_BIN} -bs --nodeps ${CHROOT_PATH}${RPM_TOPDIR}/SPECS/${SPEC}" 2>&1 \
		| grep "No such file or directory" \
		| perl -p -e "s#.*/##g" \
		| cut -d ":" -f1 \
		| head -n1`
		while [ "${NOSRC_FILE}" != "" ] \
		&& [ "${EXIT}" != "1" ]; do
			if [ -f "${LBD}/NOSRC/${NOSRC_FILE}" ]; then
				cp -v ${LBD}/NOSRC/${NOSRC_FILE} ${CHROOT_PATH}${RPM_TOPDIR}/SOURCES/
				NOSRC_FILE=`${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
					su -l ${LOGNAME} \
					-c "${RPM_REBUILD_BIN} -bs --nodeps ${CHROOT_PATH}${RPM_TOPDIR}/SPECS/${SPEC}" 2>&1 \
				| grep "No such file or directory" \
				| perl -p -e "s#.*/##g" \
				| cut -d ":" -f1 \
				| head -n1`
			else
				echo "`date -R` ${pkg} missing file: ${NOSRC_FILE}" >> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
				EXIT=1
			fi
		done
		unset NOSRC_FILE
	fi

	SRPMS=`${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
		su -l ${LOGNAME} \
		-c "${RPM_REBUILD_BIN} -bs --nodeps ${RPM_TOPDIR}/SPECS/${SPEC} ${RPM_OPTIONS}" 2> /dev/null | cut -d " " -f2`

	echo "*** Installing BuildRequires for: ${pkg} ***"

	if	[ "${INSTALLER}" = "apt" ]; then
		${SUDO_BIN} ${APT_BIN} \
			build-dep \
			${CHROOT_PATH}/${SRPMS} \
			--assume-yes \
			--no-remove \
			-o RPM::RootDir=${CHROOT_PATH} \
			>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1

	elif	[ "${INSTALLER}" = "smart" ]; then
		for i in `rpm -qpR ${CHROOT_PATH}/${SRPMS} \
			| grep -v "rpmlib(CompressedFileNames)"`; do
			${SUDO_BIN} ${SMART_BIN} \
				-o force-channels=rpm-sys,${MEDIA} \
				-o rpm-root=${CHROOT_PATH} \
				install \
				-y \
				${i} \
				>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1
		done

	elif	[ "${INSTALLER}" = "urpmi" ]; then

		${SUDO_BIN} ${URPMI_BIN} -q -p \
			${CHROOT_PATH}/${SRPMS} \
			--auto \
			--media ${MEDIA} \
			--nolock \
			--no-uninstall \
			--no-verify-rpm \
			--root ${CHROOT_PATH} \
			--split-length 0 \
			--strict-arch \
			>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1

	elif	[ "${INSTALLER}" = "yum" ]; then
		echo "*** not implemented yet ***"
		${SUDO_BIN} ${YUM_BIN} \
			-c ${YUM_CONFIG} \
			--installroot=${CHROOT_PATH}
	fi


# Start building
	echo "*** Building: ${pkg} ***"

# Rebuild the (no)src.rpm, write output to file.
	${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
	su -l ${LOGNAME} \
	-c "${RPM_BIN} -qa | sort" \
	> ${BUILD_OUTPUT}/rpm_qa/${pkg}

	${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
	su -l ${LOGNAME} \
	-c "${RPM_REBUILD_BIN} --rebuild \
       	/var/tmp/${pkg}.${SRC_EXT}.rpm \
	${RPM_OPTIONS}" \
	>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1

	rm -fr ${CHROOT_PATH}/var/tmp/${pkg}.${SRC_EXT}.rpm

# Analyze the buildoutput, and move it to the right directory.

# Buildoutput end with "+ exit 0", build is successful.
	if [ "$(tail --lines 1 ${BUILD_OUTPUT}/BO/${pkg})" = "+ exit 0" ]; then
# Find the "Wrote:" statements in the buildoutput.
		for n in `fgrep Wrote: ${BUILD_OUTPUT}/BO/${pkg} | cut -b 8- `; do

			o=`echo "${n}"	| perl -p -e "s#.*/##g" 2> /dev/null`

# Dependency loop
			for i in `${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
			su -l ${LOGNAME} \
			-c "${RPM_BIN} -qp --provides ${n} | cut -d \" \" -f1"`; do
				${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
				su -l ${LOGNAME} \
				-c "${RPM_BIN} -q ${i} \
				--whatprovides \
				| grep -v \"no package provides\""
			done >> ${BUILD_OUTPUT}/deploop/${pkg}

			if [ -e ${COMPARE}/${o} ]; then

# urpmdiff
				if [ "$(urpmdiff -d ${COMPARE}/${o} ${CHROOT_PATH}${n} | wc -l)" != "2" ]; then
					urpmdiff -d ${COMPARE}/${o} ${CHROOT_PATH}${n} >> ${BUILD_OUTPUT}/urpmdiff_d/${pkg}
				fi

				if [ "$(urpmdiff -fm ${COMPARE}/${o} ${CHROOT_PATH}${n} | wc -l)" != "2" ]; then
					urpmdiff -fm ${COMPARE}/${o} ${CHROOT_PATH}${n} >> ${BUILD_OUTPUT}/urpmdiff_fm/${pkg}
				fi

# Rpmlint
				RPMLINT_NEW_FILE="${HOME}/tmp/${o}.RPMLINT_NEW.`hostname -f`"
				rpmlint ${CHROOT_PATH}${n} \
					| grep -v "no-packager-tag" \
					| grep -v "invalid-buildhost" \
					| grep -v "no-signature" \
					| sort -u > ${RPMLINT_NEW_FILE}
				RPMLINT_OLD_FILE="${HOME}/tmp/${o}-RPMLINT_OLD.`hostname -f`"
				rpmlint ${COMPARE}/${o} \
					| sort -u > ${RPMLINT_OLD_FILE}
				if [ `diff ${RPMLINT_OLD_FILE} ${RPMLINT_NEW_FILE} | wc -l` != "0" ]; then
					echo "Rpmlint ${o}" >> ${BUILD_OUTPUT}/rpmlint/${pkg}
					diff -uN ${RPMLINT_OLD_FILE} ${RPMLINT_NEW_FILE} \
					>> ${BUILD_OUTPUT}/rpmlint_diff/${pkg}
				fi
				rm -fr	${RPMLINT_NEW_FILE} \
			       		${RPMLINT_OLD_FILE}
				unset	RPMLINT_NEW_FILE \
			       		RPMLINT_OLD_FILE
			fi
			rpmlint ${CHROOT_PATH}${n} \
			| grep -v "no-packager-tag" \
			| grep -v "invalid-buildhost" \
			| grep -v "no-signature" \
			>> ${BUILD_OUTPUT}/rpmlint/${pkg}
			n_srpms=`rpm -qp --queryformat %{SOURCERPM} ${CHROOT_PATH}${n} \
				| perl -p -e "s#\.nosrc\.rpm##g" \
				| perl -p -e "s#\.src\.rpm##g"`
			if	[ "${n_srpms}" != "${pkg}" ]; then
				echo "srpm mismatch `echo ${CHROOT_PATH}${n} | perl -p -e "s#.*/##g"` ${n_srpms} ${pkg}" \
				>> ${BUILD_OUTPUT}/srpms_mismatch/${pkg}
			fi
		done
		for i in deploop rpmlint ; do
			if [ `cat	${BUILD_OUTPUT}/${i}/${pkg}	| wc -l` = 0 ]; then
				rm -fr	${BUILD_OUTPUT}/${i}/${pkg}
			fi
		done
# Optional: Find the Redundant BuildRequires
		if	[ "${REDUNDANT}" = 1 ] && \
			[ "${INSTALLER}" = "urpmi" ]; then
			echo "*** Computing Redundant BuildRequires for: ${pkg} ***"
			RBR_DIR=" ${HOME}/tmp/${pkg}.`hostname -f`"
			mkdir ${RBR_DIR}
	
			for a in basesystem rpm-build `rpm -qpR ${CHROOT_PATH}/${SRPMS} \
				| cut -d " " -f1 \
				| grep -v rpmlib \
				| sort -u`; do
				for b in `urpmq -pr ${a} --media ${MEDIA} 2> /dev/null \
					| perl -p -e "s#\|#\n#g" 2> /dev/null \
					| grep -v "no package named ${a}" \
					| cut -d ":" -f2 \
					| sort -u`; do
					for c in `urpmq -dpr ${b} --media ${MEDIA} 2> /dev/null \
						| perl -p -e "s#\|#\n#g" 2> /dev/null \
						| sort -u`; do
						if [ "${b}" != "${c}" ]; then
							echo "${c}"
						fi
					done
				done | sort -u > ${RBR_DIR}/${a}
			done

# Find the obselete BuildRequires: 
			for a in `rpm -qpR ${CHROOT_PATH}/${SRPMS} \
				| cut -d " " -f1 \
				| grep -v rpmlib \
				| sort -u`; do
				u=0
				for b in `urpmq -pr ${a} --media ${MEDIA} 2> /dev/null \
					| perl -p -e "s#\|#\n#g" 2> /dev/null \
					| grep -v "no package named ${a}" \
					| cut -d ":" -f2 \
					| sort -u`; do
					for c in `cat ${RBR_DIR}/* | sort -u`; do
						if [ "${b}" = "${c}" ]; then
							u=1
						fi
					done
				done
				if [ "${u}" = 1 ]; then
					echo "@@@ Redundant BuildRequires: ${a} @@@" \
					`fgrep ${b} ${RBR_DIR}/* \
					| cut -d ":" -f1 \
					| perl -p -e "s#.*/##g" 2> /dev/null \
					| perl -p -e "s#\n# #g" 2> /dev/null`
				fi
			done \
			> ${BUILD_OUTPUT}/redundant/${pkg} 2>&1
			rm -fr ${RBR_DIR}
			unset RBR_DIR

			if [ `cat	${BUILD_OUTPUT}/redundant/${pkg}	| wc -l` = 0 ]; then
				rm -fr	${BUILD_OUTPUT}/redundant/${pkg}
			fi
		fi

		BUILD_RESULT=OK

# "error: cannot open" file does not exist --> remove the buildoutput.
	elif	[ "$(tail -n1 ${BUILD_OUTPUT}/BO/${pkg} \
		| cut -b 0-18)" = "error: cannot open" ]; then
		rm -fr ${BUILD_OUTPUT}/BO/${pkg}

# "Architecture excluded" --> arch_excl
	elif	[ "$(tail -n2 ${BUILD_OUTPUT}/BO/${pkg} | head -n1 \
		| cut -b 8-19)" = Architecture ]; then
		BUILD_RESULT=arch_excl

# "source package expected, binary found" --> broken
	elif	[ "$(tail -n2 ${BUILD_OUTPUT}/BO/${pkg} | head -n1 \
		| cut -b 0-37)" = "source package expected, binary found" ]; then
		BUILD_RESULT=broken

# "cannot be installed" --> cannot_be_installed
	elif	[ "$(fgrep -c "error: /var/tmp/${pkg}.${SRC_EXT}.rpm cannot be installed" \
		${BUILD_OUTPUT}/BO/${pkg})" != "0" ]; then
		BUILD_RESULT=cannot_be_installed

# "Package already exists: %package debug" --> debug
	elif	[ `fgrep -c "error: Package already exists: %package debug" \
		${BUILD_OUTPUT}/BO/${pkg}` != "0" ]; then
		BUILD_RESULT=debug

# error while ordering dependencies --> rpm_order
	elif	[ `grep -c "error while ordering dependencies" ${BUILD_OUTPUT}/BO/${pkg}` != "0" ]; then
		BUILD_RESULT=rpm_order

# When a "no package named" error shows up, there is an issue with the chroot.
# Move the file to the rebuild directory and stop lbd.
	elif	[ `grep -c "no package named" ${BUILD_OUTPUT}/BO/${pkg}` != "0" ]; then
		BUILD_RESULT=rebuild
		rm -fr ${LBD}/run/${LOGNAME}@`hostname -f`

# Missing Dependency --> dependency
	elif	[ `grep -c "is needed by" ${BUILD_OUTPUT}/BO/${pkg}` != "0" ]; then
		BUILD_RESULT=dependency

# "File not found" --> file_not_found
	elif	[ "$(tail -n1 ${BUILD_OUTPUT}/BO/${pkg} \
		| cut -b 5-18)" = "File not found" ]; then
		BUILD_RESULT=file_not_found
		if	[ "${KEEP_BUILDDIR}" = "1" ]; then
			KEEP_BUILDDIR=2
		fi

# "The following files ought to be marked as %multiarch" --> multiarch
	elif	[ `fgrep -c "The following files ought to be marked as %multiarch" \
		${BUILD_OUTPUT}/BO/${pkg}` != "0" ]; then
		BUILD_RESULT=multiarch

# "Bad exit status" --> problem
	elif	[ "$(tail -n1 ${BUILD_OUTPUT}/BO/${pkg} \
		| cut -b 5-19)" = "Bad exit status" ]; then

#	Also see if an "Internal compiler error" took place.
#	If so, make a copy of the Buildoutput
		if	[ `fgrep -c "Internal compiler error" ${BUILD_OUTPUT}/BO/${pkg}` != "0" ]; then
			echo "`date -R` Internal compiler error ${pkg}" \
			>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
		fi

#	Also see if a "Segmentation fault" took place.
#	If so, make a copy of the Buildoutput
		if	[ `fgrep -c "Segmentation fault" ${BUILD_OUTPUT}/BO/${pkg}` != "0" ]; then
			echo "`date -R` Segmentation fault ${pkg}" \
			>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
		fi

		BUILD_RESULT=problem

# "Installed (but unpackaged) file(s) found:" --> unpackaged_files
	elif	[ `fgrep -c "Installed (but unpackaged) file(s) found:" \
		${BUILD_OUTPUT}/BO/${pkg}` != "0" ]; then
		BUILD_RESULT=unpackaged_files
		if	[ "${KEEP_BUILDDIR}" = "1" ]; then
			KEEP_BUILDDIR=2
		fi
	fi

if [ "${KEEP_BUILDDIR}" = "2" ]; then
	DATE=`date +%Y-%m-%d_%H:%M:%S`
	pushd ${CHROOT_PATH}${RPM_TOPDIR}/BUILD
	tar cf	${BUILD_OUTPUT}/KEEP_BUILDDIR/${pkg}.tar *
	bzip2	${BUILD_OUTPUT}/KEEP_BUILDDIR/${pkg}.tar
	popd
fi

unset KEEP_BUILDDIR

# Move resulting binary packages.
	if	[ "${KEEP_RPMS}" = "1" ]; then
		mv -f	${CHROOT_PATH}${RPM_TOPDIR}/RPMS/${P_ARCH}/* \
			${CHROOT_PATH}${RPM_TOPDIR}/RPMS/noarch/* \
			${BUILD_OUTPUT}/RPMS/ 2> /dev/null
	fi

# Remove BuildRequires from chroot
	echo "*** Removing BuildRequires for: ${pkg} ***"

	END_FILE="${HOME}/tmp/END.`hostname -f`"
	${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
	su -l ${LOGNAME} \
	-c "${RPM_BIN} -qa | sort" \
	> ${END_FILE}
	end=`cat ${END_FILE} | wc -l`

	a=0
	LIMIT=3
	while	[ "${begin}" != "${end}" ] \
	&&	[ "${a}" -le "${LIMIT}" ]; do
		let "a+=1"
		R=`diff ${BEGIN_FILE} ${END_FILE}  \
		| grep "^>" \
		| cut -d " " -f2 \
		| perl -p -e "s#\n# #g"`

		if [ "${a}" -lt "2" ]; then
			echo "*** Removing BuildRequires: ${R}" \
			>> ${BUILD_OUTPUT}/BO/${pkg}
			${SUDO_BIN} ${RPM_BIN} -e ${R} \
			--allmatches \
			--root ${CHROOT_PATH} \
			>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1
		elif [ "${a}" -eq "2" ]; then
			echo "*** Rebuilding rpmdb, Removing BuildRequires: ${R}" \
			>> ${BUILD_OUTPUT}/BO/${pkg}
			echo "`date -R` begin: ${begin} != end: ${end} try ${a}" \
			>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
			${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} rm -fr /var/lib/rpm/__db.00*
			${SUDO_BIN} ${RPM_BIN} --rebuilddb \
			--root ${CHROOT_PATH} 2>&1

			${SUDO_BIN} ${RPM_BIN} -e ${R} \
			--allmatches \
			--root ${CHROOT_PATH} \
			>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1
		elif [ "${a}" -gt "2" ]; then
			echo "*** Removing BuildRequires with --noscripts: ${R}" \
			>> ${BUILD_OUTPUT}/BO/${pkg}
			echo "`date -R` begin: ${begin} != end: ${end} try ${a}" \
			>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
			${SUDO_BIN} ${RPM_BIN} -e ${R} \
			--allmatches \
			--noscripts \
			--root ${CHROOT_PATH} \
			>> ${BUILD_OUTPUT}/BO/${pkg} 2>&1
		fi

		${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} \
		su -l ${LOGNAME} \
		-c "${RPM_BIN} -qa | sort" \
		> ${END_FILE}
		end=`cat ${END_FILE} | wc -l`
	done
	unset a LIMIT

	if [ "$begin" != "${end}" ]; then
		echo "`date -R` begin: ${begin} != end: ${end}" \
		>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
		rm -fr ${LBD}/run/${LOGNAME}@`hostname -f`
	fi

# Unmount /proc, /dev, /dev/pts, /sys & /proc/bus/usb
	${SUDO_BIN} /bin/umount ${CHROOT_PATH}/proc/bus/usb
	${SUDO_BIN} /bin/umount ${CHROOT_PATH}/sys
	${SUDO_BIN} /bin/umount ${CHROOT_PATH}/dev/pts
	${SUDO_BIN} /bin/umount ${CHROOT_PATH}/dev
	${SUDO_BIN} /bin/umount ${CHROOT_PATH}/proc

# If BUILD_RESULT is unknow then there must be a problem
	if [ "${BUILD_RESULT}" = "" ]; then
		BUILD_RESULT="problem"
	fi

# Move the buildoutput to the right directory
	mv ${BUILD_OUTPUT}/BO/${pkg} \
		${BUILD_OUTPUT}/BO/${BUILD_RESULT}/
	echo "`date -R` Build result for: ${pkg} was: ${BUILD_RESULT}" \
	>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt

	unset BUILD_RESULT

# remove some files left over after removing packages
# packages may be fixed with rpm 4.4 requires(...) ?
	${SUDO_BIN} ${CHROOT_BIN} ${CHROOT_PATH} rm -fr /etc/sgml

cleanup

else
	rm -fr ${BUILD_OUTPUT}/.searching
fi

rm -fr	${BEGIN_FILE} \
	${END_FILE}

# unset environment variable, so they won't be re-used in the next loop
unset	\
	BEGIN_FILE \
	BUILD_OUTPUT \
	CHROOT_PATH \
	COMPARE \
	END_FILE \
	INSTALLER \
	KEEP_BUILDDIR \
	KEEP_RPMS \
	MEDIA \
	RPM_MACROS \
	RPM_SRC
}

update_media () {
# Check if media needs updating
# Input:
# - media
# Output: none
# Configuration file ${config_file}
# - CHROOT_PATH
# - GENHDLIST_BIN
# - MEDIA_PATH
# - REGENERATE
# - RPMS_PATH (?)
# - UPDATE

media=${1}

. ${config_file} media_${media}

MEDIA_FILE="${LBD}/media/${media}.`hostname -f`"

if [ -e ${MEDIA_FILE} ]; then
	if [ `stat -c %Y ${MEDIA_PATH}` \
		!= `cat ${MEDIA_FILE}` ]; then
		if [ "${REGENERATE}" = "1" ]; then
			echo "`date -R` regenerating media ${media}" \
			>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt

			pushd `dirname ${MEDIA_PATH}`
			${GENHDLIST_BIN} "${RPMS_PATH}" 2>/dev/null
			popd

			stat -c %Y \
			`dirname ${MEDIA_PATH}`/${RPMS_PATH} \
			> ${MEDIA_FILE}

			unset REGENERATE
		fi

		echo "`date -R` updating media ${media}" \
		>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
		if	[ "${INSTALLER}" = "apt" ]; then
			echo "warning: using experimental apt support"
			${SUDO_BIN} ${APT_BIN} \
				update
			if [ "${UPDATE}" = 1 ]; then
				. ${config_file} ${CHROOT_PATH}
				echo "`date -R` upgrading chroot ${CHROOT_PATH}" \
				>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
				${SUDO_BIN} ${APT_BIN} \
				dist-upgrade \
				--assume-yes \
				-o RPM::RootDir=${CHROOT_PATH}
			fi
		elif	[ "${INSTALLER}" = "smart" ]; then
			${SUDO_BIN} ${SMART_BIN} \
				-o force-channels=rpm-sys,${media} \
				update
			if [ "${UPDATE}" = "1" ]; then
				. ${config_file} ${CHROOT_PATH}
				echo "`date -R` upgrading chroot ${CHROOT_PATH}" \
				>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
				${SUDO_BIN} ${SMART_BIN} \
					-o force-channels=rpm-sys,${CHROOT_MEDIA} \
					-o rpm-root=${CHROOT_PATH} \
					upgrade \
					-y
			fi

		elif	[ "${INSTALLER}" = "urpmi" ]; then
			echo "@@@ Going for: SUDO_BIN=${SUDO_BIN} URPMI_UPDATE_BIN=${URPMI_UPDATE_BIN} media=${media} @@@"
			${SUDO_BIN} ${URPMI_UPDATE_BIN} ${media} --no-md5sum
			if	[ "${UPDATE}" = "1" ]; then
				for CHROOT_PATH_ in `echo ${CHROOT_PATH} | perl -p -e "s#,#\n#g" 2> /dev/null`; do
					. ${config_file} ${CHROOT_PATH_}
					echo "`date -R` upgrading chroot ${CHROOT_PATH_}" \
					>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
					echo "@@@ Going for: SUDO_BIN=${SUDO_BIN} URPMI_BIN=${URPMI_BIN} CHROOT_MEDIA=${CHROOT_MEDIA} CHROOT_PATH=${CHROOT_PATH_} @@@"
					${SUDO_BIN} ${URPMI_BIN} -q \
					--auto-select \
					--auto \
					--keep \
					--media ${CHROOT_MEDIA} \
					--nolock \
					--no-uninstall \
					--no-verify-rpm \
					--root ${CHROOT_PATH_} \
					--split-length 0 \
					--strict-arch \
					>> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
				done
			fi
		elif	[ "${INSTALLER}" = "yum" ]; then
			${YUM_BIN} \
			-c ${YUM_CONFIG} \
			--install-root=${CHROOT_PATH} \
			update
		else
			echo "@@@ ERROR, no INSTALLER selected @@@"
		fi

		echo "@@@ Updating ${MEDIA_FILE} @@@"
		stat -c %Y \
		${MEDIA_PATH} \
		> ${MEDIA_FILE}

		continue
	fi
else
	echo "1" > ${MEDIA_FILE}
	continue 2
fi
}

# Main loop

DONE=0

if [ $# -eq "0" ]; then
	echo "Usage: `basename $0` options (-c)"
	exit $E_OPTERROR        # Exit and explain usage, if no argument(s) given.
fi  

while getopts ":c:" Option ; do
	case $Option in
	c) config_file=${OPTARG};;
	*) echo "*** Unimplemented option chosen ***";;
	esac
done

shift $(($OPTIND - 1))

if [ ! -e "${config_file}" ]; then
	echo "*** Missing config file ***"
	break
else
	. ${config_file}
fi

check_config

unset OLD_TARGET
if [ "${BUILD_OLD}" != "no" ]; then
	OLD_TARGET=old
fi

while	[ -e ${LBD}/run/${LOGNAME}@`hostname -f` ] ; do
#echo "A"
	for i in `${config_file} media`; do
#echo "B i=${i}"
		update_media ${i}
	done
	for new_old in rebuild new dependency ${OLD_TARGET} ; do
#echo "C new_old=${new_old}"
		for i in `${config_file} src`; do
#echo "D i=${i}"
			new_job=`find_${new_old} ${i}`
#echo "E new_job=${new_job}"
			if  [ "${new_job}" != "" ]; then
#echo "F new_job=${new_job}"
				build ${new_job} ${i}
				DONE=1
				continue 4
			fi
#echo "G"
		done
#echo "H"
	done
	break
done

if [ "${DONE}" = "1" ]; then
	echo "`date -R` done" >> ${LBD}/log/${LOGNAME}@`hostname -f`.`date +%Y-%m`.txt
fi
